{include for unzip.pas: General functions used by both inflate and explode}

{C code by info-zip group, translated to Pascal by Christian Ghisler}
{based on unz51g.zip}

{*********************************** CRC Checking ********************************}

procedure UpdateCRC(var s:iobuf;len:integer);
var i:integer;
begin
{$ifndef assembler}
 for i:=0 to len-1 do begin
    { update running CRC calculation with contents of a buffer }
    crc32val:=crc_32_tab[(byte(crc32val) xor s[i]) and $ff] xor (crc32val shr 8);
  end;
{$else}
  asm
    les di,s
    mov ax,li.lo(crc32val)
    mov dx,li.hi(crc32val)
    mov si,offset crc_32_tab      {Segment remains DS!!!}
    mov cx,len
    or cx,cx
    jz @finished
@again:
    mov bl,al           {byte(crcval)}
    mov al,ah           {shift DX:AX by 8 bits to the right}
    mov ah,dl
    mov dl,dh
    xor dh,dh

    xor bh,bh
    xor bl,es:[di]  {xor s^}
    inc di
    shl bx,1            {Offset: Index*4}
    shl bx,1
    xor ax,[si+bx]
    xor dx,[si+bx+2]
    dec cx
    jnz @again
@finished:
    mov li.lo(crc32val),ax
    mov li.hi(crc32val),dx
  end;
{$endif}
end;

{************************ keep other programs running ***************************}

procedure messageloop;
{$ifdef windows}
var msg:tmsg;
begin
  lastusedtime:=gettickcount;
  while PeekMessage(Msg,0,0,0,PM_Remove) do
    if (dlghandle=0) or not IsDialogMessage(dlghandle,msg) then begin
      TranslateMessage(Msg);
      DispatchMessage(Msg);
    end;
end;
{$else}
var ch:word;
begin
  if keypressed then begin
    ch:=byte(readkey);
    if ch=0 then ch:=256+byte(readkey);  {Extended code}
    if ch=dlgnotify then totalabort:=true;
  end
end;
{$endif}

{************************* tell dialog to show % ******************************}

{$ifdef windows}
procedure showpercent;
var percent:word;
begin
  if compsize<>0 then begin
    percent:=reachedsize*100 div compsize;
    if percent>100 then percent:=100;
    if (percent<>oldpercent) then begin
      oldpercent:=percent;
      if dlghandle<>0 then begin     {Use dialog box for aborting}
        {Sendmessage returns directly -> ppercent contains result}
        sendmessage(dlghandle,wm_command,dlgnotify,longint(@percent));
        totalabort:=(percent=$FFFF);   {Abort pressed!}
      end else
        if dlgnotify<>0 then
          totalabort:=getasynckeystate(dlgnotify)<0;  {break Key pressed!}
    end;
  end;
end;
{$endif}

{************************** fill inbuf from infile *********************}

procedure readbuf;
begin
  if reachedsize>compsize+2 then begin {+2: last code is smaller than requested!}
    readpos:=sizeof(inbuf); {Simulates reading -> no blocking}
    zipeof:=true
  end else begin
    messageloop;      {Other programs, or in DOS: keypressed?}
    {$ifdef windows}
    showpercent;      {Before, because it shows the data processed, not read!}
    {$endif}
    {$I-}
    blockread(infile,inbuf,sizeof(inbuf),readpos);
    {$I+}
    if (ioresult<>0) or (readpos=0) then begin  {readpos=0: kein Fehler gemeldet!!!}
      readpos:=sizeof(inbuf); {Simulates reading -> CRC error}
      zipeof:=true;
    end;
    inc(reachedsize,readpos);
    dec(readpos);    {Reason: index of inbuf starts at 0}
  end;
  inpos:=0;
end;

{**** read byte, only used by explode ****}

procedure READBYTE(var bt:byte);
begin
  if inpos>readpos then readbuf;
  bt:=inbuf[inpos];
  inc(inpos);
end;

{*********** read at least n bits into the global variable b *************}

procedure NEEDBITS(n:byte);
var nb:longint;
begin
{$ifndef assembler}
  while k<n do begin
    if inpos>readpos then readbuf;
    nb:=inbuf[inpos];
    inc(inpos);
    b:=b or nb shl k;
    inc(k,8);
  end;   
{$else}
  asm
    mov si,offset inbuf
    mov ch,n
    mov cl,k
    mov bx,inpos    {bx=inpos}
@again:
    cmp cl,ch
    JAE @finished   {k>=n -> finished}
    cmp bx,readpos
    jg @readbuf     
@fullbuf:
    mov al,[si+bx]  {dx:ax=nb}
    xor ah,ah
    xor dx,dx
    cmp cl,8      {cl>=8 -> shift into DX or directly by 1 byte}
    JAE @bigger8
    shl ax,cl     {Normal shifting!}
    jmp @continue
@bigger8:
    mov di,cx     {save cx}
    mov ah,al     {shift by 8}
    xor al,al
    sub cl,8      {8 bits shifted}
@rotate:
    or cl,cl
    jz @continue1 {all shifted -> finished}
    shl ah,1      {al ist empty!}
    rcl dx,1
    dec cl
    jmp @rotate
@continue1:
    mov cx,di
@continue:
    or li.hi(b),dx {b=b or nb shl k}
    or li.lo(b),ax
    inc bx         {inpos}
    add cl,8       {inc k by 8 Bits}
    jmp @again

@readbuf:     
    push si
    push cx
    call readbuf   {readbuf not critical, called only every 2000 bytes}
    pop cx
    pop si
    mov bx,inpos   {New inpos}
    jmp @fullbuf

@finished:
    mov k,cl
    mov inpos,bx
  end;
{$endif}
end;

{***************** dump n bits no longer needed from global variable b *************}

procedure DUMPBITS(n:byte);
begin
{$ifndef assembler}
  b:=b shr n;
  k:=k-n;
{$else}
  asm
    mov cl,n
    mov ax,li.lo(b)
    mov dx,li.hi(b)

    mov ch,cl
    or ch,ch
    jz @finished
@rotate:
    shr dx,1           {Lower Bit in Carry}
    rcr ax,1
    dec ch
    jnz @rotate
@finished:
    mov li.lo(b),ax
    mov li.hi(b),dx
    sub k,cl
  end;
{$endif}
end;

{********************* Flush w bytes directly from slide to file ******************}
function flush(w:word):boolean;  
var n:word;          {True wenn OK}

begin
  {$I-}
  blockwrite(outfile,slide[0],w,n);
  {$I+}
  flush:=(n=w) and (ioresult=0);  {True-> alles ok}
  UpdateCRC(iobuf(pointer(@slide[0])^),w);
end;

{******************************* Break string into tokens ****************************}

VAR
  _Token: PChar;

FUNCTION StrTok(Source: PChar; Token: CHAR): PChar;
  VAR P: PChar;
BEGIN
  IF Source <> Nil THEN _Token := Source;
  IF _Token = Nil THEN begin
    strTok:=nil;
    exit
  end;
  P := StrScan(_Token, Token);
  StrTok := _Token;
  IF P <> Nil THEN BEGIN
    P^ := #0;
    Inc(P);
  END;
  _Token := P;
END;
